%
% highpas performs a highpass operation on xy 

%***********************************************************************************
%
%                              Jim Merriam
%                       University of Saskatchewan
%                          jim.merriam@usask.ca
%                              (Feb, 1995)  
%                            (last April 1999)
%
%***********************************************************************************
%
figure(6),clf
fig6=figure(6);
set(fig6,'name','HIGH PASS FILTER')  % create new figure in figure 5


contour(FX,FY,abs(FZ),50),title('FOURIER TRANSFORM OF GRID')
xlabel('Fx (cy/unit)'),ylabel(' Fy (cy/unit)')
set(gca,'color',[.75 .75 .75]);

%%%%%%%%%%%%%%%%%%%%%% GUI BLOCK FOR HELP %%%%%%%%%%%%%%%%%%%%%%%%
HELP6=['Define the shoulders of the cutoffs in x and y frequencies by positioning  ', ... 
      'the pointer at the lower limits (third quadrant first, first quadrant second)', ...
     'and clicking. The shoulder will be outlined in green. Then define the ramp by clicking ',  ...
     ' in the same way. The ramp will  be outlined in red.  ', ... 
     '(The shoulder is outside the ramp)  After viewing click OK.'];
HelpOff6=['set(helpboxh6,''visible'',''off'')'];
HelpHighPass=['helpboxh6=uicontrol(''style'',''text'',''units'',''normalized'',', ... 
' ''HorizontalAlignment'',''left'',''backgroundcolor'',[1 1 0],', ...
' ''position'',[.5 .8 .5 .2],''string'',HELP6);'];
helpmenu6=uimenu(fig6,'Label','HELP','position',2); % button to turn help on
uimenu(helpmenu6,'Label','HELP ON','Callback','eval(HelpHighPass)','position',1);
uimenu(helpmenu6,'Label','HELP OFF','CALLBACK','eval(HelpOff6)','position',2);

%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%

hradio6=uicontrol('style','text','string','ZOOM IN?','units','normalized','position',[.77 .93 .11 .05]);
hradio6_1=uicontrol('style','radiobutton','string','OK','units','normalized','position',[.88 .93 .07 .05]);
hradio6_2=uicontrol('style','radiobutton','string','YES','units','normalized','position',[.7 .93 .08 .05]);
% wait for a selection from uicontrol
 while get(hradio6_1,'value')==0 %
  if(get(hradio6_2,'value')==1);
  current_axis=axis;
  axis(current_axis/2);
  end
  pause(2)
  end




%%%%%%%%%%%%%%%%%%% BLOCK TO SET CUTOFFS %%%%%%%%%%%%%%%%%%%%%%%%%%
[x_cutoff,y_cutoff]=ginput(2);    % ginput for frequency cutoffs and plot
hold on
plot([x_cutoff(1) x_cutoff(2) x_cutoff(2) x_cutoff(1) x_cutoff(1)], ... 
   [y_cutoff(1) y_cutoff(1) y_cutoff(2) y_cutoff(2)  y_cutoff(1)],'g--','LineWidth',2);    

[x_ramp,y_ramp]=ginput(2);  % ginput for ramp frequency and plot
plot([x_ramp(1) x_ramp(2) x_ramp(2) x_ramp(1) x_ramp(1)], ... 
   [y_ramp(1) y_ramp(1) y_ramp(2) y_ramp(2)  y_ramp(1)],'r--','LineWidth',2);

  if(x_cutoff(2)<x_ramp(2)) % shoulder < ramp switch
  disp('SWITCHING SHOULDER AND RAMP')
  temp=x_cutoff;
  x_cutoff=x_ramp;
  x_ramp=temp;
  temp=y_cutoff;
  y_cutoff=y_ramp;
  y_ramp=temp;
  plot([x_ramp(1) x_ramp(2) x_ramp(2) x_ramp(1) x_ramp(1)], ... 
   [y_ramp(1) y_ramp(1) y_ramp(2) y_ramp(2)  y_ramp(1)],'r--','LineWidth',2);
  plot([x_cutoff(1) x_cutoff(2) x_cutoff(2) x_cutoff(1) x_cutoff(1)], ... 
   [y_cutoff(1) y_cutoff(1) y_cutoff(2) y_cutoff(2)  y_cutoff(1)],'g--','LineWidth',2);    
  end
%%%%%%%%%%%%%%%%%%%%%%%%%  END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
%%%%%%%%%%%%%%%%%%%%% BLOCK TO PAUSE FOR OK %%%%%%%%%%%%%%%%%%%%%
OKButton=['set(hOK,''visible'',''off''),PauseForOK=''n'';'];
hOK=uimenu('label','OK','position',3,'Callback',OKButton);
PauseForOK='y';
while PauseForOK=='y'  %    
pause(.1)
end
clear OKButton hOK
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%% BLOCK TO DEFINE FILTER %%%%%%%%%%%%%%%%%%%%%%%%%
N_rampx=floor(abs(x_ramp(2)-x_cutoff(2))/dfx);
N_rampy=floor(abs(y_ramp(2)-y_cutoff(2))/dfy);
if N_rampx <=2; N_rampx=2;end
if N_rampy <=2; N_rampy=2;end
% set up default filter fez with ones and zeros
fez=ones(N,M); % set up default filter
 I=find(reshape(FX,N*M,1)<=x_cutoff(2)-N_rampx*dfx & reshape(FX,N*M,1)>=x_cutoff(1)+N_rampx*dfx);
fez=reshape(fez,N*M,1);
fez(I)=zeros(size(I));
J=find(reshape(FY,N*M,1)<=y_cutoff(2)-N_rampy*dfy & reshape(FY,N*M,1)>=y_cutoff(1)+N_rampy*dfy);
fez(J)=zeros(size(J));
fez=reshape(fez,N,M);


% ramp the x edges of the filter
i=find(FX(1,:)<=x_cutoff(2) & FX(1,:)>=x_cutoff(2)-N_rampx*dfx);
j=find(FX(1,:)>=x_cutoff(1) & FX(1,:)<=x_cutoff(1)+N_rampx*dfx);
rampx=[0:length(i)-1]/length(i);
rampx=(cos(rampx*pi-pi)+1)/2;
fez(:,i)=(ones(N,1)*rampx).*fez(:,i);
rampx=[length(j)-1:-1:0]/length(j);
rampx=(cos(rampx*pi-pi)+1)/2;
fez(:,j)=(ones(N,1)*rampx).*fez(:,j);

% ramp the y edges of the filter 
i=find(FY(:,1)<=y_cutoff(2) & FY(:,1)>=y_cutoff(2)-N_rampy*dfy);
j=find(FY(:,1)>=y_cutoff(1) & FY(:,1)<=y_cutoff(1)+N_rampy*dfy);
rampy=[0:length(i)-1]/length(i);
rampy=(cos(rampy*pi-pi)+1)/2;

fez(i,:)=(ones(M,1)*rampy)'.*fez(i,:);
rampy=[length(j)-1:-1:0]/length(j);
rampy=(cos(rampy*pi-pi)+1)/2;
fez(j,:)=(ones(M,1)*rampy)'.*fez(j,:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% mesh the filter
hold off,
mesh(FX,FY,fez),title('HIGH PASS FILTER TO BE APPLIED')
xlabel('Fx (cy/unit)'),ylabel(' Fy (cy/unit)')
set(gca,'color',[.75 .75 .75]);

%%%%%%%%%%%%%%%%%%%%% BLOCK TO PAUSE FOR OK %%%%%%%%%%%%%%%%%%%%%
OKButton=['set(hOK,''visible'',''off''),PauseForOK=''n'';'];
hOK=uimenu('label','OK','position',3,'Callback',OKButton);
PauseForOK='y';
while PauseForOK=='y'  %    
pause(.1)
end
clear OKButton
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%


% apply the filter
Z_fil=fez.*FZ;

% mesh the filter applied to transform
mesh(FX,FY,abs(Z_fil)),title('HIGH PASS FILTER APPLIED TO TRANSFORM')
xlabel('Fx (cy/unit)'),ylabel(' Fy (cy/unit)')
set(gca,'color',[.75 .75 .75]);


%%%%%%%%%%%%%%%%%%%%%%%% QUIT BLOCK  %%%%%%%%%%%%%%%%%%%%%%%%%% 
QuitButtonhp=['set(hQuithp,''visible'',''off''),QuitHighpass=''y'';'];
hQuithp=uimenu('label','QUIT','position',3,'Callback',QuitButtonhp);
QuitHighpass='n';
while QuitHighpass=='n'  %    
pause(.1)
end
clear QuitButtonhp hQuithp
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%

if CallFrom=='bps'
figure(fig7)
else
figure(mainfig)
end

 if exist('helpboxh6')==1,set(helpboxh6,'visible','off'),end
%%%%%%%%%%%%%%%%%%%%%%%%%%% CLEAN UP %%%%%%%%%%%%%%%%%%%%%%%%%%% 
clear QuitHighpass fig6 HELP6 temp x_cutoff x_ramp y_cutoff ... 
y_ramp rampx rampy  OKButton  ...
PauseForOK  I J N_rampx N_rampy helpboxh6 j HelpOff6 ...
HelpHighPass hOK