%
% laplace
%
% 2D Potential fields filtering
% The following must exist in the workspace
%
% x_in - grid
% y_in - grid
% z_in - gridded values
% dx - sampling in x
% dy - sampling in y
% z_out is created

%***********************************************************************************
%
%                              Jim Merriam
%                       University of Saskatchewan
%                          jim.merriam@usask.ca
%                              (Feb, 1995)  
%                           (last April 1999)
%
%***********************************************************************************
%

% GREETING BLOCK
% clear laplace variables
clear z_out z_outx z_outy CallFromHorizDeriv
clear HelpMain finalfig mainfig hmain HELPM GetRidOf N_band M_band FX FY FZ ...
N_in M_in N M z_work fx fy  Z_fil rmsi  htext ib keepon mainmenu perim s fez fezh fezl ...
CallFrom CallTo dfx dfy FX_max FY_max FX_min FY_min rmsr HelpOff helpboxM QuitFinalMap  ...
ZZ damnNaN notNaN  hradio5 hradio5_1 hradio5_2 hradio6 hradio6_1 hradio6_2  ...
hradio7 hradio7_1 hradio7_2 hradio8 hradio8_1 hradio8_2 hradio10 hradio10_1 hradio10_2 current_axis  ...
S polynomialsurface A i XX fez fr pr HelpFinal FIGAXES TEXTAXES HelpOffFinal temp fu FZ ...
CALLBACK5 CallFrom HELP7 HELPFinal HelpBandPas HelpOff7 HelpRadspec HelpSpecFac finalMap ...
helpmenu helpmenu10 helpmenu2 helpmenu5 helpmenu6 helpmenu7 quitlaplace z_outl z_outh ...
QuitFirstvd fig7 finalmap helpmenu POSIE N_in z_nan FZ Z_filx Z_fily Z_outx Z_outy 

close all; % 
CallTo='xxx';
CallFrom='xxx';
LaplaceGreeting
pause(1)
close(figure(1))
finalfig=figure(14); % create final figure window
set(finalfig,'name','LAPLACE OUTPUT FIGURE')


%%%%%%%%%%%%%%%%%%% THIS BLOCK DRAWS THE RAW MAP %%%%%%%%%%%%%%%%%%%%%%%
mainfig=figure(1);  % create first figure window
set(mainfig,'name','LAPLACE MAIN MENU')
hmain=axes('position',[.1 .1 .8 .8]);
mesh(x_in,y_in,z_in),xlabel('X'),ylabel('Y'),title('RAW GRID')
N_band=9;  % number of rows to add
M_band=9;  % number of columns to add
[N_in,M_in]=size(z_in); % original size
N=N_in;
M=M_in;
%%%%%%%%%%%%%%%%%%%%%%THIS BLOCK REPLACES NaNs WITH NEAREST NEIGHBOUR
  ZZ=reshape(z_in,N*M,1); %
  XX=reshape(x_in,N*M,1); %
  YY=reshape(y_in,N*M,1); %
  
  damnNaN=find(isnan(ZZ));
  notNaN=find(~isnan(ZZ));
    for j=1:length(damnNaN)
    distance=(XX(damnNaN(j))-XX).^2+(YY(damnNaN(j))-YY).^2;
    m=find(distance==0);distance(m)=max(distance); % set distance =0 to max   
      for l=1:length(distance)
      k=find(distance==min(distance));
        if isnan(ZZ(k(1)));
        distance(k(1))=distance(k(1))*2;
        elseif ~isnan(ZZ(k(1)));
        ZZ(damnNaN(j))=ZZ(k(1));break
        end% end if
      end % end for
    end % end for

  z_work=reshape(ZZ,N,M); % create working grid 
  ZZ(notNaN)=1;
  ZZ(damnNaN)=NaN;
  z_nan=reshape(ZZ,N,M); % create grid of NaN's
  S=sprintf('%5d NaN''s FOUND ',length(damnNaN))';
  disp(S')
  disp('WARNING NaNs IN INPUT GRID REPLACED WITH NEAREST NEIGHBOURS')
  disp('MASK of NaNS created in z_nan ')
  clear XX YY ZZ damnNaN notNaN k l j distance m  
  %%%%%%% THIS BLOCK REMOVES A LINEAR REGIONAL FIT TO THE PERIMETER %%%%%%%
  %%%%%%% TO LESSEN GIBBS                                           %%%%%%% 
   ZZ=reshape([z_work(:,1)' z_work(:,M)' z_work(1,:) z_work(N,:)]',2*N+2*M,1);
   XX=reshape([x_in(:,1)' x_in(:,M)' x_in(1,:) x_in(N,:)]',2*N+2*M,1);
   YY=reshape([y_in(:,1)' y_in(:,M)' y_in(1,:) x_in(N,:)]',2*N+2*M,1);
   A=[ones(size(XX)) XX YY];   
   polynomialsurface=A\ZZ;
   
   XX=reshape(x_in,N*M,1);
   YY=reshape(y_in,N*M,1);
   A=[ones(size(XX)) XX YY];   

   Z_REG=A*polynomialsurface; 

   Z_REG=reshape(Z_REG,N,M); %Regional grid
   z_work=z_work-Z_REG;
   disp(' A PLANE FIT TO THE PERIMETER HAS BEEN SUBTRACTED FROM THE WHOLE GRID ')
   clear X YY ZZ
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% Subtract the perimeter average from the whole grid.
perim=sum([z_work(:,1)' z_work(:,M)' z_work(1,:) z_work(N,:)])/(2*N+2*M);
z_work=z_work-perim;
%%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%% GUI BLOCK FOR MENUS %%%%%%%%%%%%%%%%%%%%%%%%
quitlaplace=uimenu(mainfig,'Label','QUIT LAPLACE','position',2);
finalmap=uimenu(mainfig,'Label','FINAL MAP','position',2);
helpmenu=uimenu(mainfig,'Label','LAPLACE HELP','position',2); % button to turn help on
mainmenu=uimenu(mainfig,'Label','   FILTERS','position',2);

uimenu(mainmenu,'Label','RADIAL FREQUENCY SPECTRUM','Callback','radspec')
uimenu(mainmenu,'Label','FIRST VERTICAL DERIVATIVE','Callback','firstvd')
uimenu(mainmenu,'Label','SECOND VERTICAL DERIVATIVE','Callback','secvd')
uimenu(mainmenu,'Label','LOW PASS','Callback','lowpas')
uimenu(mainmenu,'Label','HIGH PASS','Callback','highpas')
uimenu(mainmenu,'Label','BAND PASS','Callback','bandpas')
uimenu(mainmenu,'Label','CONTINUATION','Callback','contin')
uimenu(mainmenu,'Label','SPECTRAL FACTORIZATION','Callback','specfac')
uimenu(mainmenu,'Label','HORIZONTAL DERIVATIVE','Callback','horizd')
uimenu(mainmenu,'Label','ANALYTIC SIGNAL','Callback','analytic')
uimenu(mainmenu,'Label','STRIKE FILTER','Callback','strikefilt')


uimenu(helpmenu,'Label','HELP ON','Callback','eval(HelpMain)','position',1)
uimenu(helpmenu,'Label','HELP OFF','CALLBACK','eval(HelpOff)','position',2)
uimenu(finalmap,'Label','FINAL MAP','CALLBACK','FinalMap','position',1)
uimenu(quitlaplace,'Label','QUIT','CALLBACK','eval(CleanUp)','position',1)

%%%%%%%%%%%%%%%%%%%%%% GUI BLOCK  FOR HELP %%%%%%%%%%%%%%%%%%%%%%%%
HelpMain=['helpboxM=uicontrol(''style'',''text'',''units'',''normalized'',', ... 
' ''HorizontalAlignment'',''left'',''backgroundcolor'',[1 1 0],', ...
' ''position'',[.7 .81 .3 .19],''string'',HELPM);';];
% help string
HELPM=[' Laplace performs filtering on 2D gridded potential fields data. ', ... 
'A SIMPLE REGIONAL SHOULD BE REMOVED FIRST. ' ...
'Select a filter, or a sequence of filters, from the FILTERS menu. ', ...
'Click FINAL MAP to draw the filtered map. Click QUIT LAPLACE to exit.'];

HelpOff=['set(helpboxM,''visible'',''off'')']; % button to turn help off


%%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK FOR HELP %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK FOR MENUS %%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%% THIS BLOCK PADS AND LINEARLY TAPERS THE PERIMETER %%%%%%%%%%%%%%

  for ib=1:N_band
  z_work(N_in+ib,:)=z_work(N_in,:)*(N_band-ib)/N_band;
  end
[N,M]=size(z_work);
z_work=flipud(z_work);
  for ib=1:N_band
z_work(N+ib,:)=z_work(N,:)*(N_band-ib)/N_band;
  end
z_work=flipud(z_work);
[N,M]=size(z_work);
  for ib=1:M_band
  z_work(:,M+ib)=z_work(:,M)*(M_band-ib)/M_band;
  end
[N,M]=size(z_work);
z_work=fliplr(z_work);
  for ib=1:M_band
  z_work(:,M+ib)=z_work(:,M)*(M_band-ib)/M_band;
  end
z_work=fliplr(z_work);
[N,M]=size(z_work);

%%%%%%%%%%%%%%%%%%%%%%%%%%    END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%


  if(floor(N/2)*2~=N) % if N is odd delete last row 
    z_work(N,:)=[];
  end

   
  if(floor(M/2)*2~=M) % if M is odd delete last column
   z_work(:,M)=[];
  end
  
  
[N,M]=size(z_work);  % final size of padded and tapered grid
fx=freq(M,dx); % frequencies in x dir
fy=freq(N,dy); % frequencies in y dir
fx=[fx' -fliplr(fx(2:length(fx)-1)')]'; % append negative frequencies
fy=[fy' -fliplr(fy(2:length(fy)-1)')]'; % append negative frequencies
FX=ones(size(fy))*fx';  % gridded frequencies
FY=fy*ones(size(fx))';  % gridded frequencies
FZ=fft2(z_work,N,M);     % 2D Fourier transform of xy

[N,M]=size(FZ); % size of input grid
FX_max=max(max(FX));
FX_min=min(min(FX)); % maximum and minimum frequencies
FY_max=max(max(FY));
FY_min=min(min(FY));
dfx=abs(FX(1,1)-FX(1,2)); % frequency increment in x
dfy=abs(FY(1,1)-FY(2,1)); % frequency increment in y

Z_fil=FZ; % default filtered = raw

CallFrom='xxx'; % initialize CallFrom


Z_REG=Z_REG+perim; % add the perimeter average to the linear regional

%%%%%%%%%%%%%%%%%%%%%%%%%%% CLEAN UP %%%%%%%%%%%%%%%%%%%%%%%%%%% 
CleanUp=['if CallFrom~=''hzd'';z_out=z_out+Z_REG+perim;end, clear HelpMain finalfig mainfig ', ...
'hmain HELPM GetRidOf N_band M_band FX FY FZ', ...
'N_in M_in N M z_work fx fy  Z_fil rmsi  htext ib keepon mainmenu s fez fezh fezl', ...
'CallFrom CallTo dfx dfy FX_max FY_max FX_min FY_min rmsr HelpOff helpboxM QuitFinalMap ', ...
'ZZ damnNaN notNaN  hradio5 hradio5_1 hradio5_2 hradio6 hradio6_1 hradio6_2 ', ...
'hradio7 hradio7_1 hradio7_2 hradio8 hradio8_1 hradio8_2 hradio10 hradio10_1 hradio10_2 current_axis ' ...
'S polynomialsurface A i XX fez fr pr HelpFinal FIGAXES TEXTAXES HelpOffFinal temp fu FZ', ...
'hradio11_1 hradio11 hradio11_2', ...
'CALLBACK5 CallFrom HELP7 HELPFinal HelpBandPas HelpOff7 HelpRadspec HelpSpecFac finalMap', ...
'helpmenu helpmenu10 helpmenu2 helpmenu5 helpmenu6 helpmenu7 quitlaplace z_outl z_outh', ...
'QuitFirstvd fig7 finalmap helpmenu POSIE N_in z_nan FZ Z_filx Z_fily Z_outx Z_outy CallFromHorizDeriv CleanUp', ...
'  hQuit QuitButton hbox11_1 hbox11_2 hbox11_3 hbox11_4 hbox11_5'];
%FX FY FZ
