%
% lowpas performs a lowpass operation on xy

%***********************************************************************************
%
%                              Jim Merriam
%                       University of Saskatchewan
%                          jim.merriam@usask.ca
%                              (Feb, 1995)  
%                            (last April 1999)
%
%***********************************************************************************
%
figure(5),clf
fig5=figure(5);
set(fig5,'name','LOW PASS FILTER')  % create new figure in figure 4

contour(FX,FY,abs(FZ),50)              % plot the contour map of Fourier transform
title('FOURIER TRANSFORM OF GRID')
set(gca,'color',[.75 .75 .75]);
xlabel('Fx (cy/unit)'),ylabel(' Fy (cy/unit)')

%%%%%%%%%%%%%%%%%%%%%% GUI BLOCK FOR HELP %%%%%%%%%%%%%%%%%%%%%%%%
HELP5=['Define the shoulders of the cutoffs in x and y frequencies by positioning  ', ... 
      'the pointer at the lower limits (third quadrant first, first quadrant second) ', ...
     'and clicking. The shoulder will be outlined in green. Then define the ramp by clicking ',  ...
     ' in the same way. The end of the ramp will  be outlined in red.  After viewing click OK ', ...
     'to return to the calling menu.'];

HelpOff5=['set(helpboxh5,''visible'',''off'')']; % button to turn help off

HelpLowPass=['helpboxh5=uicontrol(''style'',''text'',''units'',''normalized'',' ... 
' ''HorizontalAlignment'',''left'',''backgroundcolor'',[1 1 0],', ...
' ''position'',[.5 .8 .5 .2],''string'',HELP5);'];
helpmenu5=uimenu(fig5,'Label','HELP','position',2); % button to turn help on
uimenu(helpmenu5,'Label','HELP ON','Callback','eval(HelpLowPass)','position',1);
uimenu(helpmenu5,'Label','HELP OFF','CALLBACK','eval(HelpOff5)','position',2);

%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%



hradio5=uicontrol('style','text','string','ZOOM IN?','units','normalized','position',[.77 .93 .11 .05]);
hradio5_1=uicontrol('style','radiobutton','string','OK','units','normalized','position',[.88 .93 .07 .05]);
hradio5_2=uicontrol('style','radiobutton','string','YES','units','normalized','position',[.7 .93 .08 .05]);
% wait for a selection from uicontrol
 while get(hradio5_1,'value')==0 %
  if(get(hradio5_2,'value')==1);
  current_axis=axis;
  axis(current_axis/2);
  end
  pause(2)
  end


%%%%%%%%%%%%%%%%%%% BLOCK TO SET CUTOFFS %%%%%%%%%%%%%%%%%%%%%%%%%%
[x_cutoff,y_cutoff]=ginput(2);    % ginput for frequency cutoffs and plot
hold on
plot([x_cutoff(1) x_cutoff(2) x_cutoff(2) x_cutoff(1) x_cutoff(1)], ... 
   [y_cutoff(1) y_cutoff(1) y_cutoff(2) y_cutoff(2)  y_cutoff(1)],'g--','LineWidth',2);    

[x_ramp,y_ramp]=ginput(2);  % ginput for ramp frequency and plot
plot([x_ramp(1) x_ramp(2) x_ramp(2) x_ramp(1) x_ramp(1)], ... 
   [y_ramp(1) y_ramp(1) y_ramp(2) y_ramp(2)  y_ramp(1)],'r--','LineWidth',2);

  if(x_cutoff(2)>x_ramp(2)) % switch if shoulder > ramp switch
  disp('SWITCHING SHOULDER AND RAMP')
  temp=x_cutoff;
  x_cutoff=x_ramp;
  x_ramp=temp;
  temp=y_cutoff;
  y_cutoff=y_ramp;
  y_ramp=temp;
  plot([x_ramp(1) x_ramp(2) x_ramp(2) x_ramp(1) x_ramp(1)], ... 
   [y_ramp(1) y_ramp(1) y_ramp(2) y_ramp(2)  y_ramp(1)],'r--','LineWidth',2);
  plot([x_cutoff(1) x_cutoff(2) x_cutoff(2) x_cutoff(1) x_cutoff(1)], ... 
   [y_cutoff(1) y_cutoff(1) y_cutoff(2) y_cutoff(2)  y_cutoff(1)],'g--','LineWidth',2);    
  end
%%%%%%%%%%%%%%%%%%%%% BLOCK TO PAUSE FOR OK %%%%%%%%%%%%%%%%%%%%%
  
OKButton=['set(hOK,''visible'',''off''),PauseForOK=''n'';'];
hOK=uimenu('label','OK','position',3,'Callback',OKButton);
PauseForOK='y';
while PauseForOK=='y'  %    
pause(.1)
end
clear OKButton hOK
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%




%%%%%%%%%%%%%%%%%%%%% BLOCK TO DEFINE FILTER %%%%%%%%%%%%%%%%%%%%%%%%%
N_rampx=floor(abs(x_ramp(2)-x_cutoff(2))/dfx);
N_rampy=floor(abs(y_ramp(2)-y_cutoff(2))/dfy);
if N_rampx <=2; N_rampx=2;end
if N_rampy <=2; N_rampy=2;end
% set up default filter fez with ones and zeros
fez=ones(N,M); % set up default filter
I=find(reshape(FX,N*M,1)>=x_cutoff(2)+N_rampx*dfx | reshape(FX,N*M,1)<=x_cutoff(1)-N_rampx*dfx);
fez=reshape(fez,N*M,1);
fez(I)=zeros(size(I));
J=find(reshape(FY,N*M,1)>=y_cutoff(2)+N_rampy*dfy | reshape(FY,N*M,1)<=y_cutoff(1)-N_rampy*dfy);
fez(J)=zeros(size(J));
fez=reshape(fez,N,M);

% ramp the x edges of the filter
i=find(FX(1,:)>=x_cutoff(2) & FX(1,:)<=x_cutoff(2)+N_rampx*dfx);
j=find(FX(1,:)<=x_cutoff(1) & FX(1,:)>=x_cutoff(1)-N_rampx*dfx);
rampx=[length(i)-1:-1:0]/length(i);
rampx=(cos(rampx*pi-pi)+1)/2;
fez(:,i)=(ones(N,1)*rampx).*fez(:,i);
rampx=[0:length(j)-1]/length(j);
rampx=(cos(rampx*pi-pi)+1)/2;
fez(:,j)=(ones(N,1)*rampx).*fez(:,j);

% ramp the y edges of the filter 
i=find(FY(:,1)>=y_cutoff(2) & FY(:,1)<=y_cutoff(2)+N_rampy*dfy);
j=find(FY(:,1)<=y_cutoff(1) & FY(:,1)>=y_cutoff(1)-N_rampy*dfy);
rampy=[length(i)-1:-1:0]/length(i);
rampy=(cos(rampy*pi-pi)+1)/2;
fez(i,:)=(ones(M,1)*rampy)'.*fez(i,:);
rampy=[0:length(j)-1]/length(j);
rampy=(cos(rampy*pi-pi)+1)/2;
fez(j,:)=(ones(M,1)*rampy)'.*fez(j,:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% mesh the filter
hold off
mesh(FX,FY,fez)
title('LOW PASS FILTER TO BE APPLIED')
xlabel('Fx (cy/unit)'),ylabel(' Fy (cy/unit)')
set(gca,'color',[.75 .75 .75]);

%%%%%%%%%%%%%%%%%%%%% BLOCK TO PAUSE FOR OK %%%%%%%%%%%%%%%%%%%%%
OKButton=['set(hOK,''visible'',''off''),PauseForOK=''n'';'];
hOK=uimenu('label','OK','position',3,'Callback',OKButton);
PauseForOK='y';
while PauseForOK=='y'  %    
pause(.1)
end
clear OKButton hOK
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%



% apply the filter
Z_fil=fez.*FZ;

% mesh the filter applied to transform
mesh(FX,FY,abs(Z_fil));
set(gca,'color',[.75 .75 .75]);
title('LOW PASS FILTER APPLIED TO TRANSFORM')
xlabel('Fx (cy/unit)'),ylabel(' Fy (cy/unit)')



%%%%%%%%%%%%%%%%%%%%%%%% QUIT BLOCK  %%%%%%%%%%%%%%%%%%%%%%%%%% 
QuitButtonlp=['set(hQuitlp,''visible'',''off''),QuitLowpass=''y'';'];
hQuitlp=uimenu('label','QUIT', ...
 'position',3,'Callback',QuitButtonlp);
QuitLowpass='n';
while QuitLowpass=='n'  %    
pause(.1)
end

    
clear QuitButtonlp hQuitlp
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%


CallFrom;
clear QuitLowpass
 if CallFrom=='fvd'
 figure(fig3)
 elseif CallFrom=='svd'
 figure(fig4)
 elseif CallFrom=='cnt'
 figure(fig9)
 elseif CallFrom=='hzd'
 figure(fig11)
 elseif CallFrom=='anl'
 figure(fig12)
 elseif CallFrom=='str'
 figure(13)
 elseif CallFrom=='bps'
 figure(6)
 else
 figure(mainfig)
 end
 if exist('helpboxh5')==1,set(helpboxh5,'visible','off'),end
 pause(1)
 figure(1)
 %%%%%%%%%%%%%%%%%%%%%%%%%%% CLEAN UP %%%%%%%%%%%%%%%%%%%%%%%%%%% 
 clear QuitLowpass  fig5 HELP5 I J N_rampx N_rampy ...
 PauseForOK  j rampx rampy x_ramp x_cutoff y_cutoff y_ramp ...
 HelpOff5 HelpLowPass helpboxh5 
 