
%
% radspec
%
% radial frequency spectrum
% The following must exist in the workspace
% FX - gridded x frequencies
% FY - gridded y freqiuencies
% FZ - gridded 2d Fourier Transform

%***********************************************************************************
%
%                              Jim Merriam
%                       University of Saskatchewan
%                          jim.merriam@usask.ca
%                              (Feb, 1995)  
%                            (last April 1999)
%
%***********************************************************************************
%
 figure(2),clf
fig2=figure(2);
set(fig2,'name','RADIAL FREQUENCY SPECTRUM')

%%%%%%%%%%%%%%%%%%%%%% GUI BLOCK FOR HELP %%%%%%%%%%%%%%%%%%%%%%%%
HELP2=['Define linear segments by clicking on the beginning and end of each segment.', ... 
' A depth will be  estimated from the slope of each segment and displayed in the figure. ', ...
'QUIT to clean up, exit from RADIAL FREQUENCY SPECTRUM and return to ', ...
' the main menu.'];

HelpRadSpec=['helpboxh2=uicontrol(''style'',''text'',''units'',''normalized'',', ... 
' ''HorizontalAlignment'',''left'',''backgroundcolor'',[1 1 0],', ...
' ''position'',[.65 .76 .35 .24],''string'',HELP2);';];
HelpOff2=['set(helpboxh2,''visible'',''off'')']; % button to turn help off

helpmenu2=uimenu(fig2,'Label','HELP','position',2); % button to turn help on
uimenu(helpmenu2,'Label','HELP ON','Callback','eval(HelpRadSpec)','position',1);
uimenu(helpmenu2,'Label','HELP OFF','CALLBACK','eval(HelpOff2)','position',2);

%%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK FOR HELP %%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%  CALCULATE AND DISPLAY RADIAL FREQ SPECTRUM %%%%%%%%%%%%%%%%%%%%

[N,M]=size(FZ);
PR=sqrt((FZ.*conj(FZ))/(N*M));  % gridded amplitude spectrum
FR=2*pi*sqrt(FY.^2 + FX.^2);  % gridded radial frequencies in 2D
dfr=(max(max(FR))-min(min(FR)))/sqrt(N*M); % radial frequency increment
FR=reshape(FR,N*M,1); % reshape to column 
PR=reshape(PR,N*M,1); % reshape to column
pr=NaN*ones(1,floor(sqrt(N*M))); % initialize pr 
fr=ones(1,floor(sqrt(N*M))); % initialize fr

%laplacelog=[laplacelog;[' RADSPEC: THERE ARE ', int2str(N*M), ' ELEMENTS IN 2D TRANSFORM']];;
  for i=1:sqrt(N*M)
  fr(i)=min(min(FR))+dfr*(i-1);
  j=find(abs(FR)>fr(i) & abs(FR)<=fr(i)+dfr);
    if(length(j)>=1)   
    pr(i)=sum(PR(j))/length(j);   % calculate (averaged) radial frequency spectrum 
    end    
 end

i=find(~isfinite(pr));
pr(i)=[];              % trim radial freqs so only finite data
fr(i)=[];
fu=fr/(2*pi); % rad freq in cy/unit

semilogy(fu,pr,'b*')
title('RADIAL FREQUENCY SPECTRUM')
xlabel('RADIAL FREQUENCY (cy/unit)'),ylabel('AMPLITUDE'),pretty
% DEPTH IS SLOPE/log10e = SLOPE/.4343 


%%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%% THIS BLOCK  DEFINES SEGMENTS AND DEPTHS%%%%%%%%%%%%

DefineSegment=['[x,y]=ginput(2);', ... 
      'i=find(fu<x(2)&fu>x(1));', ...                        %
      'if(x(2)<x(1)),i=find(fu<x(1)&fu>x(2));end,', ...      % 
  'c=polyfit(fr(i),log10(pr(i)),1);', ...                    %
  'DEPTH=c(1)/log10(exp(1));', ...
  ' hold on,', ...
  'semilogy(fu(i),10.^polyval(c,fr(i)),''-'');', ...
  'S=sprintf(''DEPTH=  %6.2f'',abs(DEPTH));', ...
  'text(mean(fu(i)),10^polyval(c,.8*mean(fr(i))),S),', ...
  'title(''RADIAL FREQUENCY SPECTRUM''),xlabel(''Radial Frequency (cy/unit)''),ylabel(''Amplitude''),', ...
  'set(gca,''color'',[.75 .75 .75]);']; 


%%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%




%%%%%%%%%%%%%%%%%%%%%%%% GUI BLOCK FOR SEGMENTS %%%%%%%%%%%%%%%%%%%%%% 

hpop=uicontrol('style','popup','STRING','1|2|3|4|5|6|7', ... 
'position',[300 250 30 30],'Callback','NumOfSegs=get(hpop,''value'');');
hpoptext=uicontrol('style','text','string','SELECT NUMBER OF SEGMENTS', ...
'position',[330 250 100 30]);

QuitRadspec='n';
clear NumOfSegs;
while ~exist('NumOfSegs') & QuitRadspec=='n',pause(1),end  % wait for NumOfSegs to be defined

if QuitRadspec=='n';
 if NumOfSegs~=0;
  for iter=1:NumOfSegs
  eval(DefineSegment)
  end
 end
end


%%%%%%%%%%%%%%%%%%%%%%%% QUIT BLOCK  %%%%%%%%%%%%%%%%%%%%%%%%%% 
QuitButtonrs=['set(hQuitrs,''visible'',''off''),QuitRadspec=''y'';'];
hQuitrs=uimenu('label','QUIT','position',3,'Callback',QuitButtonrs);
QuitRadspec='n';
while QuitRadspec=='n'  %    
pause(.1)
end
clear QuitButtonrs hQuitrs
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%


set(hpop,'visible','off');,set(hpoptext,'visible','off');   % clean up

clear hpop hpoptext DefineSegment HelpRadspec  ...
 NumOfSegs HELP1 HELP2 QuitRadspec 


figure(mainfig)
 if exist('helpboxh2')==1,set(helpboxh2,'visible','off'),end
%%%%%%%%%%%%%%%%%%%%%%%%%%% CLEAN UP %%%%%%%%%%%%%%%%%%%%%%%%%%% 
clear PR FR  DefineSegment  HelpRadSpec HELP2 ...
hpop hpoptext NumOfSegs DEPTH c dfr fig2 iter j x y QuitRadspec S ...
HelpOff2 helpboxh2 
