
%
% specfac
%
% SPECTRAL FACTORIZATION
% The following must exist in the workspace
% FX - gridded x frequencies
% FY - gridded y freqiuencies
% FZ - gridded 2d Fourier Transform

%***********************************************************************************
%
%                              Jim Merriam
%                       University of Saskatchewan
%                          jim.merriam@usask.ca
%                              (Feb, 1995)  
%                            (last April 1999)
%
%***********************************************************************************
%



%%%%%%%%%%%%%%%%%%%  CALCULATE AND DISPLAY RADIAL FREQ SPECTRUM %%%%%%%%%%%%%%%%%%%%
CallTo='spf'; % initialize
figure(10),clf
fig10=figure(10);
set(fig10,'name','SPECTRAL FACTORIZATION')
[N,M]=size(FZ);
PR=sqrt((FZ.*conj(FZ))/(N*M));  % gridded amplitude spectrum
FR=2*pi*sqrt(FY.^2 + FX.^2);  % gridded radial frequencies in 2D
dfr=(max(max(FR))-min(min(FR)))/sqrt(N*M); % radial frequency increment
FR=reshape(FR,N*M,1); % reshape to column 
PR=reshape(PR,N*M,1); % reshape to column
pr=NaN*ones(1,floor(sqrt(N*M))); % initialize pr 
fr=ones(1,floor(sqrt(N*M))); % initialize fr

HELP10=[' Chose a suitable frequency to define the cutoff between ', ... 
      'a high frequency factor and a low frequency factor, and use ', ...
      'this as a guide in selecting fx and fy cutoffs in the 2D grid.'];
HelpOff10=['set(helpboxh10,''visible'',''off'')']; % button to turn help off
HelpSpecFac=['helpboxh10=uicontrol(''style'',''text'',''units'',''normalized'',', ... 
' ''HorizontalAlignment'',''left'',''backgroundcolor'',[1 1 0],', ...
' ''position'',[.5 .86 .5 .14],''string'',HELP10);'];
helpmenu10=uimenu(fig10,'Label','HELP','position',2); % button to turn help on
uimenu(helpmenu10,'Label','HELP ON','Callback','eval(HelpSpecFac)','position',1);
uimenu(helpmenu10,'Label','HELP OFF','CALLBACK','eval(HelpOff10)','position',2);

%%%%%%%%%%%%%%%%%%%%% BLOCK TO TURN OFF HELP %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%


  for i=1:sqrt(N*M)
  fr(i)=min(min(FR))+dfr*(i-1);
  j=find(abs(FR)>fr(i) & abs(FR)<=fr(i)+dfr);
    if(length(j)>=1)   
    pr(i)=sum(PR(j))/length(j);   % calculate (averaged) radial frequency spectrum 
    end    
 end

i=find(~isfinite(pr));
pr(i)=[];              % trim radial freqs so only finite data
fr(i)=[];


semilogy(fr/(2*pi*1.414),pr,'*')
title('RADIAL FREQUENCY SPECTRUM')
xlabel('RADIAL FREQUENCY (cy/unit)'),ylabel('AMPLITUDE'),pretty



%%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%% BLOCK TO PAUSE FOR OK %%%%%%%%%%%%%%%%%%%%%
  
OKButton=['set(hOK,''visible'',''off''),PauseForOK=''n'';'];
hOK=uimenu('label','OK','position',2,'Callback',OKButton);
PauseForOK='y';
while PauseForOK=='y'  %    
pause(.1)
end
clear OKButton hOK 

clf(figure(10))
fig10=figure(10);
set(fig10,'name','SPECTRAL FACTORIZATION')
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%

contour(FX,FY,abs(FZ),50)              % plot the contour map of Fourier transform
title('FOURIER TRANSFORM OF GRID')
set(gca,'color',[.75 .75 .75]);
xlabel('Fx (cy/unit)'),ylabel(' Fy (cy/unit)')

hradio10=uicontrol('style','text','string','ZOOM IN?','units','normalized','position',[.77 .93 .11 .05]);
hradio10_1=uicontrol('style','radiobutton','string','OK','units','normalized','position',[.88 .93 .07 .05]);
hradio10_2=uicontrol('style','radiobutton','string','YES','units','normalized','position',[.7 .93 .08 .05]);
% wait for a selection from uicontrol
 while get(hradio10_1,'value')==0 %
  if(get(hradio10_2,'value')==1);
  current_axis=axis;
  axis(current_axis/2);
  end
  pause(2)
  end

%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%% BLOCK TO SET CUTOFFS %%%%%%%%%%%%%%%%%%%%%%%%%%
[x_cutoff,y_cutoff]=ginput(2);    % ginput for frequency cutoffs and plot
hold on
plot([x_cutoff(1) x_cutoff(2) x_cutoff(2) x_cutoff(1) x_cutoff(1)], ... 
   [y_cutoff(1) y_cutoff(1) y_cutoff(2) y_cutoff(2)  y_cutoff(1)],'b--','LineWidth',2);    
%%%%%%%%%%%%%%%%%%%%% BLOCK TO PAUSE FOR OK %%%%%%%%%%%%%%%%%%%%%
  
OKButton=['set(hOK,''visible'',''off''),PauseForOK=''n'';'];
hOK=uimenu('label','OK','position',2,'Callback',OKButton);
PauseForOK='y';
while PauseForOK=='y'  %    
pause(.1)
end
clear OKButton hOK
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%% BLOCK TO DEFINE LOW BAND %%%%%%%%%%%%%%%%%%%%%%%%%
N_rampx=4;
N_rampy=4;
% set up default filter fez with ones and zeros
fezl=ones(N,M); % set up default filter
I=find(reshape(FX,N*M,1)>=x_cutoff(2)+N_rampx*dfx | reshape(FX,N*M,1)<=x_cutoff(1)-N_rampx*dfx);
fezl=reshape(fezl,N*M,1);
fezl(I)=zeros(size(I));
J=find(reshape(FY,N*M,1)>=y_cutoff(2)+N_rampy*dfy | reshape(FY,N*M,1)<=y_cutoff(1)-N_rampy*dfy);
fezl(J)=zeros(size(J));
fezl=reshape(fezl,N,M);

% ramp the x edges of the filter
i=find(FX(1,:)>=x_cutoff(2) & FX(1,:)<=x_cutoff(2)+N_rampx*dfx);
j=find(FX(1,:)<=x_cutoff(1) & FX(1,:)>=x_cutoff(1)-N_rampx*dfx);
rampx=[length(i)-1:-1:0]/length(i);
fezl(:,i)=(ones(N,1)*rampx).*fezl(:,i);
rampx=[0:length(j)-1]/length(j);
fezl(:,j)=(ones(N,1)*rampx).*fezl(:,j);

% ramp the y edges of the filter 
i=find(FY(:,1)>=y_cutoff(2) & FY(:,1)<=y_cutoff(2)+N_rampy*dfy);
j=find(FY(:,1)<=y_cutoff(1) & FY(:,1)>=y_cutoff(1)-N_rampy*dfy);
rampy=[length(i)-1:-1:0]/length(i);
fezl(i,:)=(ones(M,1)*rampy)'.*fezl(i,:);
rampy=[0:length(j)-1]/length(j);
fezl(j,:)=(ones(M,1)*rampy)'.*fezl(j,:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%BLOCK TO DEFINE HIGH BAND%%%%%%%%%%%%%%%%%%%%%%%
N_rampx=4;
N_rampy=4;
% set up default filter fez with ones and zeros
fezh=ones(N,M); % set up default filter
 I=find(reshape(FX,N*M,1)<=x_cutoff(2)-N_rampx*dfx & reshape(FX,N*M,1)>=x_cutoff(1)+N_rampx*dfx);
fezh=reshape(fezh,N*M,1);
fezh(I)=zeros(size(I));
J=find(reshape(FY,N*M,1)<=y_cutoff(2)-N_rampy*dfy & reshape(FY,N*M,1)>=y_cutoff(1)+N_rampy*dfy);
fezh(J)=zeros(size(J));
fezh=reshape(fezh,N,M);


% ramp the x edges of the filter
i=find(FX(1,:)<=x_cutoff(2) & FX(1,:)>=x_cutoff(2)-N_rampx*dfx);
j=find(FX(1,:)>=x_cutoff(1) & FX(1,:)<=x_cutoff(1)+N_rampx*dfx);
rampx=[0:length(i)-1]/length(i);
fezh(:,i)=(ones(N,1)*rampx).*fezh(:,i);
rampx=[length(j)-1:-1:0]/length(j);
fezh(:,j)=(ones(N,1)*rampx).*fezh(:,j);

% ramp the y edges of the filter 
i=find(FY(:,1)<=y_cutoff(2) & FY(:,1)>=y_cutoff(2)-N_rampy*dfy);
j=find(FY(:,1)>=y_cutoff(1) & FY(:,1)<=y_cutoff(1)+N_rampy*dfy);
rampy=[0:length(i)-1]/length(i);

fezh(i,:)=(ones(M,1)*rampy)'.*fezh(i,:);
rampy=[length(j)-1:-1:0]/length(j);
fezh(j,:)=(ones(M,1)*rampy)'.*fezh(j,:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clf(figure(10))
fig10=figure(10);
set(fig10,'name','SPECTRAL FACTORIZATION')

%clear helpboxh10
hreg=axes('position',[.07 .07 .47 .47]);
hres=axes('position',[.47 .47 .47 .47]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%BLOCK SHOWING FILTERS %%%%%%%%%%%%%%
%mesh(FX,FY,fezl)
%title('LOW PASS FILTER')
%xlabel('Fx (cy/unit)'),ylabel(' Fy (cy/unit)')
%set(gca,'color',[.75 .75 .75]);
%mesh(FX,FY,fezh)
%title('HIGH PASS FILTER')
%xlabel('Fx (cy/unit)'),ylabel(' Fy (cy/unit)')
%set(gca,'color',[.75 .75 .75]);


%pause(2)
clear OKButton hOK
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%

Z_filh=fezh.*FZ;
Z_fill=fezl.*FZ;
%axes(hreg);
%mesh(FX,FY,abs(Z_fill));
%set(gca,'color',[.75 .75 .75]);
%title('LOW PASSED SPECTRUM')
%xlabel('Fx (cy/unit)'),ylabel(' Fy (cy/unit)')
%axes(hres);
%mesh(FX,FY,abs(Z_filh));
%set(gca,'color',[.75 .75 .75]);
%title('HIGH PASSED SPECTRUM')
%xlabel('Fx (cy/unit)'),ylabel(' Fy (cy/unit)')



z_outl=ifft2(Z_fill); % inverse Fourier transform

%%%%%%%%%%% THIS BLOCK REMOVES THE PADDING %%%%%%%%%%%%%%%%
  z_outl(1:N_band,:)=[];
  z_outl=flipud(z_outl);
    if(floor(N_in/2)*2~=(N_in))
    z_outl(1:N_band-1,:)=[];
    else
    z_outl(1:N_band,:)=[];
    end
  z_outl=flipud(z_outl);
  z_outl(:,1:M_band)=[];
  z_outl=fliplr(z_outl);
    if(floor(M_in/2)*2~=M_in)
    z_outl(:,1:M_band-1)=[];
    else
    z_outl(:,1:M_band)=[];
    end
  z_outl=fliplr(z_outl);
  rmsil=sqrt(sum(sum(imag(z_outl).*imag(z_outl)))/(N_in*M_in));
  rmsrl=sqrt(sum(sum(real(z_outl).*real(z_outl)))/(N_in*M_in));

disp(['ROOT MEAN SQUARE OF IMAGINARY OUTPUT (LOW) GRID ',num2str(rmsil)])
disp(['RATIO OF RMS(IMAGINARY) TO RMS(REAL) ',num2str(rmsil/rmsrl)])
    
  z_outl=real(z_outl);    
%%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%

z_outh=ifft2(Z_filh); % inverse Fourier transform
%%%%%%%%%%% THIS BLOCK REMOVES THE PADDING %%%%%%%%%%%%%%%%
  z_outh(1:N_band,:)=[];
  z_outh=flipud(z_outh);
    if(floor(N_in/2)*2~=(N_in))
    z_outh(1:N_band-1,:)=[];
    else
    z_outh(1:N_band,:)=[];
    end
  z_outh=flipud(z_outh);
  z_outh(:,1:M_band)=[];
  z_outh=fliplr(z_outh);
    if(floor(M_in/2)*2~=M_in)
    z_outh(:,1:M_band-1)=[];
    else
    z_outh(:,1:M_band)=[];
    end
  z_outh=fliplr(z_outh);
  rmsih=sqrt(sum(sum(imag(z_outh).*imag(z_outh)))/(N_in*M_in));
  rmsrh=sqrt(sum(sum(real(z_outh).*real(z_outh)))/(N_in*M_in));

disp(['ROOT MEAN SQUARE OF IMAGINARY OUTPUT (HIGH) GRID ',num2str(rmsih)])
disp(['RATIO OF RMS(IMAGINARY) TO RMS(REAL) ',num2str(rmsih/rmsrh)])
    
  z_outh=real(z_outh);    
%%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%

axes(hreg)
mesh(x_in,y_in,z_outl),title('LOW FREQ FACTOR')
xlabel('X'),ylabel('Y')
set(gca,'color',[.75 .75 .75]);
axes(hres)
mesh(x_in,y_in,z_outh),title('HIGH FREQ FACTOR')
xlabel('X'),ylabel('Y')
set(gca,'color',[.75 .75 .75]);



%%%%%%%%%%%%%%%%%%%%%%%% QUIT BLOCK  %%%%%%%%%%%%%%%%%%%%%%%%%% 
QuitButtonspf=['set(hQuitspf,''visible'',''off''),QuitSpecfac=''y'';'];
hQuitspf=uimenu('label','QUIT','position',1,'Callback',QuitButtonspf);
QuitSpecfac='n';
while QuitSpecfac=='n'  %    
pause(.1)
end
clear QuitButtonspf hQuitspf
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(mainfig)
 if exist('helpboxh10')==1,set(helpboxh10,'visible','off'),end

%%%%%%%%%%%%%%%%%%%%%%%%%%% CLEAN UP %%%%%%%%%%%%%%%%%%%%%%%%%%% 
clear I J N_rampx N_rampy pauseFOrOK Z_filh Z_fill dfr fezh fezl ... 
helpboxh10 rampx rampy rmsih rmsil rmsrh rmsrl x_cutoff y_cutoff ...
hreg hres QuitSpecfac HelpOff10 FR PR P fig10 ans fr j pr ... 
PauseForOK HelpSpecfac HELP10