%
% strike filter for laplace
%
% strike filt may add a DC offset
%

%***********************************************************************************
%
%                              Jim Merriam
%                       University of Saskatchewan
%                          jim.merriam@usask.ca
%                              (Feb, 1998)  
%                            (last April 2000)
%
%***********************************************************************************
%

% STRIKE FILTER (GRID MUST HAVE dx=dy)
figure(13),clf
fig13=figure(13);
set(fig13,'name','STRIKE FILTER')  % create new figure in figure 13

%%%%%%%%%%%%%%%%%%%%% HELP BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
HELP13=['The azimuth and width describe the pass zone for the strike.  ', ... 
      ' The strike azimuth is measured clockwise from the top of the map (deg).', ...
     ' The width is the full width of the pass zone (deg). '];
HelpOff13=['set(helpboxh13,''visible'',''off'')']; % button to turn help off
HelpStrike=['helpboxh13=uicontrol(''style'',''text'',''units'',''normalized'',', ... 
' ''HorizontalAlignment'',''left'',''backgroundcolor'',[1 1 0],', ...
' ''position'',[.5 .85 .5 .15],''string'',HELP13);'];
uimenu(fig13,'Label','HELP ON','position',1,'Callback',HelpStrike) % button to turn help on
uimenu(fig13,'Label','HELP OFF','position',2,'Callback',HelpOff13); % button to turn help off
%%%%%%%%%%%%%%%%%%%%% BLOCK TO TURN OFF HELP %%%%%%%%%%%%%%%%%%%%%
HelpOff13=['set(helpboxh13,''visible'',''off'')'];
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%% QUERY BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%
hradio6=uicontrol('style','text','string','LOW PASS FILTER?','position',[340 340,100,40]);
hradio6_1=uicontrol('style','radiobutton','string','YES','position',[300,360,40 20]);
hradio6_2=uicontrol('style','radiobutton','string','NO','position',[300,340,40 20]);

hbox13_1=uicontrol('style','text','string','STRIKE AZIMUTH (deg)','position',[130 350,120,20]);
hbox13_2=uicontrol('style','edit','string',' ','position',[100 350,30,20], ... 
 'Callback','StrikeAz=-str2num(get(hbox13_2,''string''));');

hbox13_3=uicontrol('style','text','string','WIDTH OF PASS (deg)','position',[130 300,120,20]);
hbox13_4=uicontrol('style','edit','string',' ','position',[100 300,30,20], ... 
 'Callback','StrikeWidth=str2num(get(hbox13_4,''string''));');



% wait for a selection from uicontrol
  while (~exist('StrikeAz','var') |  ~exist('StrikeWidth','var'));
  pause(.1)
  end

% wait for a selection from uicontrol
  while get(hradio6_1,'value')==0 & get(hradio6_2,'value')==0;
  pause(.1)
  end
   
  if(get(hradio6_1,'value')==1);
  CallFrom='str';                  % if YES (lowpass filter) do it  
  lowpas; 
  else 
      Z_fil=ones(size(FZ));% else not 
  end
  
set(hradio6,'visible','off')
set(hradio6_1,'visible','off')  % turn off uicontrols
set(hradio6_2,'visible','off')
set(hbox13_1,'visible','off')
set(hbox13_2,'visible','off')
set(hbox13_3,'visible','off')
set(hbox13_4,'visible','off')
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%% BLOCK TO CALCULATE FILTER %%%%%%%%%%%%%%%%%%%
[N,M]=size(FZ);
F=atan2(FY,FX)*180/pi;
Z_stk=zeros(size(FX));
[i,j]=find(F>StrikeAz-StrikeWidth/2 & F<StrikeAz+StrikeWidth/2 ... 
   | (F>StrikeAz-StrikeWidth/2-180 & F<StrikeAz+StrikeWidth/2-180) ... 
   | (F>StrikeAz-StrikeWidth/2+180 & F<StrikeAz+StrikeWidth/2+180));
m=length(i);
   for ii=1:m
   Z_stk(i(ii),j(ii))=1;
   end
Z_fil=Z_fil.*Z_stk;
smoothie=[[0.025 0.025 0.025];[0.025 0.8 0.025];[0.025 0.025 0.025]]; % smoothing convolution filter
Z_fil=conv2(smoothie,Z_fil);% Smooth Z_fil
Z_fil(N+2,:)=[];
Z_fil(:,M+2)=[]; % delete first and last rows and columns
Z_fil(1,:)=[];
Z_fil(:,1)=[];
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%% DISPLAY BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%
mesh(FX,FY,abs(Z_fil));               % and display
set(gca,'color',[.75 .75 .75]);
xlabel('Fx (cy/unit)'),ylabel(' Fy (cy/unit)')

% write appropriate title
  if get(hradio6_2,'value') == 1
  title('STRIKE FILTER (NO LOW PASS FILTER)')
  elseif get(hradio6_1,'value') == 1
     title('STRIKE FILTER (LOW PASS FILTER APPLIED)')
  else 
  disp('PROBLEM: NO RESOLVED CHOICE FOR TITLE IN STRIKEFILT')
  end
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%
  

%%%%%%%%%%%%%%%%%%%%%%%% QUIT BLOCK  %%%%%%%%%%%%%%%%%%%%%%%%%% 
QuitButtonskf=['set(hQuitskf,''visible'',''off''),QuitStrike=''y'';'];
hQuitskf=uimenu('label','QUIT','position',1,'Callback',QuitButtonskf);
QuitStrike='n';
while QuitStrike=='n'  %    
pause(.1)
end
clear QuitButtonskf hQuitskf
%%%%%%%%%%%%%%%%%%%%%%%%%% END BLOCK %%%%%%%%%%%%%%%%%%%%%%%%%%%%
 figure(mainfig)
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%% CLEAN UP %%%%%%%%%%%%%%%%%%%%%%%%%%% 
 clear QuitStrike  hradio6 hradio6_1 hradio6_2 hbox13_1
 clear hbox13_2  hbox13_3 hbox13_4 StrikeAz Strikewidth HELP13 HelpOff13 HelpStrike
 clear helpBoxh13 m ii j StrikeWidth Z_stk F fig13 smoothie
